@@ -27,16 +27,28 @@ import os
 
 import bzrlib
 from bzrlib.commands import plugin_cmds
-from bzrlib.directory_service import directories
+from bzrlib.directory_service import (
+    AliasDirectory,
+    directories,
+    )
 
 from info import (
     bzr_plugin_version as version_info,
     )
 
 
+try:
+    from bzrlib.i18n import load_plugin_translations
+except ImportError: # No translations for bzr < 2.5
+    gettext = lambda x: x
+else:
+    translation = load_plugin_translations("bzr-builddeb")
+    gettext = translation.ugettext
+
 commands = {
-        "bd_do": [],
+        "builddeb_do": ["bd-do"],
         "builddeb": ["bd"],
+        "get_orig_source": [],
         "dep3_patch": [],
         "dh_make": ["dh_make"],
         "import_dsc": [],
@@ -56,6 +68,8 @@ def global_conf():
     from bzrlib.config import config_dir
     return os.path.join(config_dir(), 'builddeb.conf')
 local_conf = os.path.join(builddeb_dir, 'local.conf')
+new_local_conf = 'debian/local.conf.local'
+new_conf = 'debian/bzr-builddeb.conf'
 
 default_build_dir = '../build-area'
 default_orig_dir = '..'
@@ -66,6 +80,20 @@ directories.register_lazy("apt:", 'bzrlib.plugins.builddeb.directory',
         'VcsDirectory',
         "Directory that uses Debian Vcs-* control fields to look up branches")
 
+branch_aliases = getattr(AliasDirectory, "branch_aliases", None)
+if branch_aliases is not None:
+    def upstream_branch_alias(b):
+        from bzrlib.plugins.builddeb.util import debuild_config
+        b.lock_read()
+        try:
+            tree = b.basis_tree()
+            config = debuild_config(tree, False)
+            return directories.dereference(config.upstream_branch)
+        finally:
+            b.unlock()
+    branch_aliases.register("upstream", upstream_branch_alias,
+        help="upstream branch (for packaging branches)")
+
 
 def debian_changelog_commit_message(commit, start_message):
     if start_message is not None:
@@ -114,7 +142,13 @@ def debian_changelog_commit(commit, start_message):
     """hooked into bzrlib.msgeditor set_commit_message.
      Set the commit message from debian/changelog and set any LP: #1234 to bug
      fixed tags."""
-    from bzrlib.plugins.builddeb.util import find_bugs_fixed
+    from bzrlib.plugins.builddeb.util import (
+        debuild_config, find_bugs_fixed)
+
+    t = commit.work_tree
+    config = debuild_config(t, False)
+    if config.commit_message_from_changelog == False:
+        return None
 
     changes = debian_changelog_commit_message(commit, start_message)
     if changes is None:
@@ -137,7 +171,7 @@ def debian_tag_name(branch, revid):
     from bzrlib.plugins.builddeb.errors import MissingChangelogError
     from bzrlib.plugins.builddeb.import_dsc import (DistributionBranch,
         DistributionBranchSet)
-    from bzrlib.plugins.builddeb.util import (debuild_config, find_changelog)
+    from bzrlib.plugins.builddeb.util import debuild_config, find_changelog
     t = branch.repository.revision_tree(revid)
     config = debuild_config(t, False)
     try:
