@@ -14,7 +14,7 @@ from bzrlib import generate_ids
 from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import NoSuchFile, BzrCommandError, NotBranchError
 from bzrlib.osutils import (pathjoin, isdir, file_iterator, basename,
-                            file_kind, splitpath, normpath)
+                            file_kind, splitpath, normpath, walkdirs)
 from bzrlib.trace import warning
 from bzrlib.transform import TreeTransform, resolve_conflicts, cook_conflicts
 from bzrlib.workingtree import WorkingTree
@@ -74,19 +74,10 @@ class DirWrapper(object):
     def __repr__(self):
         return 'DirWrapper(%r)' % self.root
 
-    def getmembers(self, subdir=None):
-        if subdir is not None:
-            mydir = pathjoin(self.root, subdir)
-        else:
-            mydir = self.root
-        for child in os.listdir(mydir):
-            if subdir is not None:
-                child = pathjoin(subdir, child)
-            fi = FileInfo(self.root, child)
-            yield fi
-            if fi.isdir():
-                for v in self.getmembers(child):
-                    yield v
+    def getmembers(self):
+        for _, dir_block in walkdirs(self.root):
+            for relpath, _, _, stat_result, _ in dir_block:
+                yield FileInfo(self.root, relpath, stat_result)
 
     def extractfile(self, member):
         return open(member.fullpath)
@@ -94,7 +85,7 @@ class DirWrapper(object):
 
 class FileInfo(object):
 
-    def __init__(self, root, filepath):
+    def __init__(self, root, filepath, stat):
         self.fullpath = pathjoin(root, filepath)
         self.root = root
         if filepath != '':
@@ -102,7 +93,6 @@ class FileInfo(object):
         else:
             self.name = basename(root)
         self.type = None
-        stat = os.lstat(self.fullpath)
         self.mode = stat.st_mode
         if self.isdir():
             self.name += '/'
@@ -252,13 +242,17 @@ def _import_archive(tree, archive_file, file_ids_from, target_tree=None):
         renames = {}
 
         # First we find the renames
+        other_trees = file_ids_from[:]
         if target_tree is not None:
+            other_trees.insert(0, target_tree)
+        for other_tree in other_trees:
             for relative_path, member in to_process:
                 trans_id = tt.trans_id_tree_path(relative_path)
                 existing_file_id = tt.tree_file_id(trans_id)
-                target_id = target_tree.path2id(relative_path)
+                target_id = other_tree.path2id(relative_path)
                 if (target_id is not None
-                    and target_id != existing_file_id):
+                    and target_id != existing_file_id
+                    and target_id not in renames):
                     renames[target_id] = relative_path
 
         # The we do the work
@@ -348,6 +342,8 @@ def _import_archive(tree, archive_file, file_ids_from, target_tree=None):
                     finally:
                         other_tree.unlock()
                 if not found:
+                    # Should this really use the trans_id as the
+                    # file_id?
                     tt.version_file(trans_id, trans_id)
             added.add(relative_path)
 
