@@ -34,10 +34,11 @@ class NoSourceDirError(BzrError):
 
 
 class MissingUpstreamTarball(BzrError):
-    _fmt = "Unable to find the needed upstream tarball: %(tarball_name)s."
+    _fmt = ("Unable to find the needed upstream tarball for package %(package)s, "
+            "version %(version)s.")
 
-    def __init__(self, tarball_name):
-        BzrError.__init__(self, tarball_name=tarball_name)
+    def __init__(self, package, version):
+        BzrError.__init__(self, package=package, version=version)
 
 
 class TarFailed(BzrError):
@@ -142,13 +143,6 @@ class UnknownVersion(BzrError):
         BzrError.__init__(self, version=version)
 
 
-class UnknownDistribution(BzrError):
-    _fmt = "Unknown distribution: %(distribution)s."
-
-    def __init__(self, distribution):
-        BzrError.__init__(self, distribution=distribution)
-
-
 class VersionNotSpecified(BzrError):
     _fmt = "You did not specify a package version."
 
@@ -170,13 +164,6 @@ class UnsupportedRepackFormat(BzrError):
         BzrError.__init__(self, location=location)
 
 
-class PristineTarError(BzrError):
-    _fmt = 'There was an error using pristine-tar: %(error)s.'
-
-    def __init__(self, error):
-        BzrError.__init__(self, error=error)
-
-
 class SharedUpstreamConflictsWithTargetPackaging(BzrError):
     _fmt = ('The upstream branches for the merge source and target have '
             'diverged. Unfortunately, the attempt to fix this problem '
@@ -184,3 +171,63 @@ class SharedUpstreamConflictsWithTargetPackaging(BzrError):
             're-run the "merge-package" command to finish. '
             'Alternatively, until you commit you can use "bzr revert" to '
             'restore the state of the unmerged branch.')
+
+
+class PerFileTimestampsNotSupported(BzrError):
+
+    _fmt = ("Per file timestamps are not supported by the "
+            "currently loaded version of bzrlib.")
+
+
+class NoPreviousUpload(BzrError):
+
+    _fmt = ("There was no previous upload to %(distribution)s.")
+
+    def __init__(self, distribution):
+        BzrError.__init__(self, distribution=distribution)
+
+
+class UnableToFindPreviousUpload(BzrError):
+
+    _fmt = ("Unable to determine the previous upload for --package-merge.")
+
+
+class InconsistentSourceFormatError(BzrError):
+
+    _fmt = ("Inconsistency between source format and version: version is "
+            "%(version_bool)snative, format is %(format_bool)snative.")
+
+    def __init__(self, version_native, format_native):
+        if version_native:
+            version_bool = ""
+        else:
+            version_bool = "not "
+        if format_native:
+            format_bool = ""
+        else:
+            format_bool = "not "
+        BzrError.__init__(self, version_bool=version_bool, format_bool=format_bool)
+
+
+class WatchFileMissing(BzrError):
+
+    _fmt = "No watch file found."
+
+
+class StrictBuildFailed(BzrError):
+
+    _fmt = ("Build refused because there are unknown files in the tree. "
+            "To list all known files, run 'bzr unknowns'.")
+
+
+class DchError(BzrError):
+    _fmt = 'There was an error using dch: %(error)s.'
+
+    def __init__(self, error):
+        BzrError.__init__(self, error=error)
+
+
+class MultipleUpstreamTarballsNotSupported(BzrError):
+
+    _fmt = ("Importing packages using source format 3.0 multiple tarballs "
+            "is not yet supported.")
