@@ -70,24 +70,30 @@ class MergePackageTests(TestCaseWithTransport):
         ubup_o, debp_n, _ubuu, _debu = self._setup_debian_upstream_newer()
         # Ubuntu upstream.
         self.assertEquals(
-            MP._latest_version(ubup_o).upstream_version, '1.2')
+            MP._latest_version(ubup_o.branch, ubup_o.last_revision()).upstream_version,
+            '1.2')
         # Debian upstream.
         self.assertEquals(
-            MP._latest_version(debp_n).upstream_version, '1.10')
+            MP._latest_version(debp_n.branch, debp_n.last_revision()).upstream_version,
+            '1.10')
 
         ubuntup, debianp = self._setup_upstreams_not_diverged()
         # Ubuntu upstream.
         self.assertEquals(
-            MP._latest_version(ubuntup).upstream_version, '1.4')
+            MP._latest_version(ubuntup.branch, ubuntup.last_revision()).upstream_version,
+            '1.4')
         # Debian upstream.
         self.assertEquals(
-            MP._latest_version(debianp).upstream_version, '2.2')
+            MP._latest_version(debianp.branch, debianp.last_revision()).upstream_version,
+            '2.2')
 
     def test__upstream_version_data(self):
         ubup_o, debp_n, _ubuu, _debu = self._setup_debian_upstream_newer()
-        vdata = MP._upstream_version_data(debp_n.branch, ubup_o.branch)
-        self.assertEquals(vdata[0][0], Version('1.10'))
-        self.assertEquals(vdata[1][0], Version('1.2'))
+        vdata = MP._upstream_version_data(debp_n.branch, debp_n.last_revision())
+        self.assertEquals(vdata[0], Version('1.10'))
+
+        vdata = MP._upstream_version_data(ubup_o.branch, ubup_o.last_revision())
+        self.assertEquals(vdata[0], Version('1.2'))
 
     def test_debian_upstream_newer(self):
         """Diverging upstreams (debian newer) don't cause merge conflicts.
@@ -126,11 +132,11 @@ class MergePackageTests(TestCaseWithTransport):
         self.assertEqual(db1.pristine_upstream_source.has_version("package", v4), False)
 
         # The ubuntu upstream branch tip.
-        ubuu_tip = ubuu.branch.revision_history()[-1]
+        ubuu_tip = ubuu.branch.last_revision()
         # The debian upstream branch tip.
-        debu_tip = debu.branch.revision_history()[-1]
+        debu_tip = debu.branch.last_revision()
         # The ubuntu packaging branch tip.
-        ubup_tip_pre_fix = ubup.branch.revision_history()[-1]
+        ubup_tip_pre_fix = ubup.branch.last_revision()
 
         # The first conflict is resolved by calling fix_ancestry_as_needed().
         upstreams_diverged, t_upstream_reverted = MP.fix_ancestry_as_needed(ubup, debp.branch)
@@ -150,7 +156,7 @@ class MergePackageTests(TestCaseWithTransport):
         self.assertEqual(db2.pristine_upstream_source.has_version("package", v4), True)
 
         # Now let's take a look at the fixed ubuntu packaging branch.
-        ubup_tip_post_fix = ubup.branch.revision_history()[-1]
+        ubup_tip_post_fix = ubup.branch.last_revision()
         ubup_parents_post_fix = ubup.branch.repository.revision_tree(ubup_tip_post_fix).get_parent_ids()
 
         # The tip of the fixed ubuntu packaging branch has 2 parents.
@@ -626,7 +632,7 @@ class MergePackageTests(TestCaseWithTransport):
 
         def tree_nick(tree):
             return str(tree)[1:-1].split('/')[-1]
-            
+
         for version, paths, utree, urevid in vdata:
             msg = ''
             if utree is not None:
@@ -641,7 +647,7 @@ class MergePackageTests(TestCaseWithTransport):
                 msg += 'Added paths: %s. ' % str(paths)
 
             commit(msg, version)
-                
+
         return tree
 
 
