@@ -25,7 +25,6 @@
 from base64 import standard_b64encode
 
 import bz2
-import gzip
 import os
 import tarfile
 import zipfile
@@ -308,7 +307,7 @@ class RecordingSource(UpstreamSource):
     def get_latest_version(self, package, current_version):
         return self._latest
 
-    def fetch_tarballs(self, package, version, target_dir):
+    def fetch_tarballs(self, package, version, target_dir, components=None):
         self._specific_versions.append((package, version, target_dir))
         if not self._succeed:
             raise PackageVersionNotPresent(package, version, self)
@@ -432,6 +431,20 @@ class UpstreamBranchSourceTests(TestCaseWithTransport):
         self.tree.commit("msg")
         self.assertEquals("2.1+bzr2", source.get_latest_version("foo", "1.0"))
 
+    def test_version_as_revisions(self):
+        revid1 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("2.1", self.tree.branch.last_revision())
+        config = DebBuildConfig(
+            [('user.conf', True), ('default.conf', False)],
+            branch=self.tree.branch)
+        source = UpstreamBranchSource(self.tree.branch,
+            {"2.1": self.tree.branch.last_revision()},
+            config=config)
+        revid2 = self.tree.commit("msg")
+        self.assertEquals(revid2,
+            source.version_as_revision("foo", "2.1+bzr2"))
+        self.assertEquals({None: revid1}, source.version_as_revisions("foo", "2.1"))
+
     def test_version_as_revision(self):
         revid1 = self.tree.commit("msg")
         self.tree.branch.tags.set_tag("2.1", self.tree.branch.last_revision())
@@ -695,18 +708,34 @@ class TestIsUpstreamTag(TestCase):
     def test_git_upstream(self):
         self.assertTrue(is_upstream_tag('upstream/2.1'))
 
+    def test_svn_upstream(self):
+        self.assertTrue(is_upstream_tag('upstream_2.1'))
+
 
 class TestUpstreamTagVersion(TestCase):
 
     def test_simple_upstream(self):
-        self.assertEqual('2.1', upstream_tag_version('upstream-2.1'))
+        self.assertEqual((None, '2.1'), upstream_tag_version('upstream-2.1'))
 
     def test_distro_upstream(self):
-        self.assertEqual('2.1',
+        self.assertEqual((None, '2.1'),
             upstream_tag_version('upstream-debian-2.1'))
 
+    def test_svn_upstream(self):
+        self.assertEqual((None, '2.1'), upstream_tag_version('upstream_2.1'))
+
     def test_git_upstream(self):
-        self.assertEqual('2.1', upstream_tag_version('upstream/2.1'))
+        self.assertEqual((None, '2.1'), upstream_tag_version('upstream/2.1'))
+
+    def test_git_upstream_component(self):
+        self.assertEqual(("lib", '2.1'), upstream_tag_version('upstream/2.1/lib'))
+
+    def test_simple_upstream_component(self):
+        self.assertEqual(("lib", '2.1'), upstream_tag_version('upstream-2.1/lib'))
+
+    def test_distro_upstream_component(self):
+        self.assertEqual(("lib", '2.1'),
+            upstream_tag_version('upstream-debian-2.1/lib'))
 
 
 class PristineTarSourceTests(TestCaseWithTransport):
@@ -728,7 +757,7 @@ class PristineTarSourceTests(TestCaseWithTransport):
 
     def test_version(self):
         self.assertEquals(['upstream-3.3', 'upstream-debian-3.3',
-            'upstream-ubuntu-3.3', 'upstream/3.3'],
+            'upstream-ubuntu-3.3', 'upstream/3.3', 'upstream_3.3'],
             self.source.possible_tag_names("3.3", component=None))
 
     def test_version_component(self):
@@ -766,14 +795,57 @@ class PristineTarSourceTests(TestCaseWithTransport):
         rev.properties["deb-pristine-delta"] = standard_b64encode("bla")
         self.assertEquals("bla", self.source.pristine_tar_delta(rev))
 
-    def test_version_as_revision_missing(self):
+    def test_version_as_revisions_missing(self):
         self.assertRaises(PackageVersionNotPresent,
-            self.source.version_as_revision, None, "1.2")
+            self.source.version_as_revisions, None, "1.2")
 
-    def test_version_as_revision(self):
+    def test_version_as_revisions_single(self):
         revid1 = self.tree.commit("msg")
         self.tree.branch.tags.set_tag("upstream-2.1", revid1)
-        self.assertEquals(revid1, self.source.version_as_revision(None, "2.1"))
+        self.assertEquals({None: revid1},
+            self.source.version_as_revisions(None, "2.1"))
+
+    def test_version_component_as_revision(self):
+        revid1 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("upstream-2.1/lib", revid1)
+        self.assertEquals(revid1,
+            self.source.version_component_as_revision(None, "2.1", "lib"))
+
+    def test_version_as_revisions(self):
+        revid1 = self.tree.commit("msg")
+        revid2 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("upstream-2.1", revid1)
+        self.tree.branch.tags.set_tag("upstream-2.1/lib", revid2)
+        self.assertEquals({ None: revid1, "lib": revid2 },
+                self.source.version_as_revisions(None, "2.1", [
+            ("upstream_2.1.orig.tar.gz", None, "somemd5sum"),
+            ("upstream_2.1.orig-lib.tar.gz", "lib", "othermd5sum")]))
+
+    def test_version_as_revisions_partially_missing(self):
+        revid1 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("upstream-2.1", revid1)
+        self.assertRaises(PackageVersionNotPresent,
+                self.source.version_as_revisions, None, "2.1", [
+            ("upstream_2.1.orig.tar.gz", None, "somemd5sum"),
+            ("upstream_2.1.orig-lib.tar.gz", "lib", "othermd5sum")])
+
+    def test_has_version_multiple(self):
+        revid1 = self.tree.commit("msg")
+        revid2 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("upstream-2.1", revid1)
+        self.tree.branch.tags.set_tag("upstream-2.1/lib", revid2)
+        self.assertTrue(
+                self.source.has_version(None, "2.1", [
+            ("upstream_2.1.orig.tar.gz", None, "somemd5sum"),
+            ("upstream_2.1.orig-lib.tar.gz", "lib", "othermd5sum")]))
+
+    def test_has_version_partially_missing(self):
+        revid1 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("upstream-2.1", revid1)
+        self.assertFalse(
+                self.source.has_version(None, "2.1", [
+            ("upstream_2.1.orig.tar.gz", None, "somemd5sum"),
+            ("upstream_2.1.orig-lib.tar.gz", "lib", "othermd5sum")]))
 
 
 class TarfileSourceTests(TestCaseWithTransport):
@@ -938,3 +1010,21 @@ class TestGatherOrigTarballs(TestCaseWithTransport):
         self.assertEquals(
             [os.path.join(self.test_dir, "mypkg_1.0.orig.tar.gz")],
             gather_orig_files("mypkg", "1.0", "."))
+
+    def test_multiple(self):
+        self.build_tree(["mypkg_1.0.orig.tar.gz", "mypkg_1.0.orig-foo.tar.gz"])
+        self.assertEquals(
+            set([os.path.join(self.test_dir, "mypkg_1.0.orig.tar.gz"),
+             os.path.join(self.test_dir, "mypkg_1.0.orig-foo.tar.gz")]),
+            set(gather_orig_files("mypkg", "1.0", ".")))
+
+    def test_utf8_invalid_file(self):
+        f = open("\xf6o.tar.gz", "w")
+        try:
+            f.write("foo")
+        finally:
+            f.close()
+        self.build_tree(["mypkg_1.0.orig.tar.gz"])
+        self.assertEquals(
+            [os.path.join(self.test_dir, "mypkg_1.0.orig.tar.gz")],
+            gather_orig_files(u"mypkg", "1.0", "."))
