@@ -64,7 +64,7 @@ def new_tarball_name(package, version, old_name):
 
 
 class UpstreamSource(object):
-    """A source for upstream versions (uscan, get-orig-source, etc)."""
+    """A source for upstream versions (uscan, debian/rules, etc)."""
 
     def get_latest_version(self, package, current_version):
         """Check what the latest upstream version is.
@@ -75,31 +75,34 @@ class UpstreamSource(object):
         """
         raise NotImplementedError(self.get_latest_version)
 
-    def version_as_revision(self, package, version):
-        """Lookup the revision id for a particular version.
+    def version_as_revisions(self, package, version, tarballs=None):
+        """Lookup the revision ids for a particular version.
 
         :param package: Package name
-        :package version: Version string
+        :param version: Version string
         :raise PackageVersionNotPresent: When the specified version was not
             found
+        :return: dictionary mapping component names to revision ids
         """
-        raise NotImplementedError(self.version_as_revision)
+        raise NotImplementedError(self.version_as_revisions)
 
-    def has_version(self, package, version, md5=None):
+    def has_version(self, package, version, tarballs=None):
         """Check whether this upstream source contains a particular package.
 
         :param package: Package name
         :param version: Version string
-        :param md5: Optional required MD5sum of the resulting tarball
+        :param tarballs: Tarballs list
         """
         raise NotImplementedError(self.has_version)
 
-    def fetch_tarballs(self, package, version, target_dir):
+    def fetch_tarballs(self, package, version, target_dir, components=None):
         """Fetch the source tarball for a particular version.
 
         :param package: Name of the package
         :param version: Version string of the version to fetch
         :param target_dir: Directory in which to store the tarball
+        :param components: List of component names to fetch; may be None,
+            in which case the backend will have to find out.
         :return: Paths of the fetched tarballs
         """
         raise NotImplementedError(self.fetch_tarballs)
@@ -113,7 +116,7 @@ class AptSource(UpstreamSource):
     """Upstream source that uses apt-source."""
 
     def fetch_tarballs(self, package, upstream_version, target_dir,
-            _apt_pkg=None):
+            _apt_pkg=None, components=None):
         if _apt_pkg is None:
             import apt_pkg
         else:
@@ -168,59 +171,77 @@ class AptSource(UpstreamSource):
         return True
 
 
-class GetOrigSourceSource(UpstreamSource):
-    """Upstream source that uses the get-orig-source rule in debian/rules."""
+class DebianRulesSource(UpstreamSource):
+    """Upstream source that uses rules in debian/rules."""
 
     def __init__(self, tree, top_level):
         self.tree = tree
         self.top_level = top_level
 
-    def _get_orig_source(self, source_dir, prefix, target_dir):
-        note("Trying to use get-orig-source to retrieve needed tarball.")
-        command = ["make", "-f", "debian/rules", "get-orig-source"]
+    def _get_rule_source(self, source_dir, rule, make_vars=None):
+        command = ["make", "-f", "debian/rules", rule]
+        if make_vars is not None:
+            command.extend(["%s=%s" % item for item in make_vars.iteritems()])
+
         proc = subprocess.Popen(command, cwd=source_dir)
         ret = proc.wait()
         if ret != 0:
-            note("Trying to run get-orig-source rule failed")
-            return None
-        filenames = []
-        for filename in os.listdir(source_dir):
-            if not filename.startswith(prefix):
-                continue
-            fetched_tarball = os.path.join(source_dir, filename)
-            if os.path.exists(fetched_tarball):
-                repack_tarball(fetched_tarball, filename,
-                               target_dir=target_dir)
-                filenames.append(os.path.join(target_dir, filename))
-        if filenames:
-            return filenames
-        note("get-orig-source did not create file with prefix %s", prefix)
-        return None
+            note("Trying to run %s rule failed" % rule)
+            return False
+        return True
 
-    def fetch_tarballs(self, package, version, target_dir):
+    def _get_current_source(self, source_dir, package, version, target_dir):
+        rule = "get-packaged-orig-source"
+        note("Trying to use %s to retrieve needed tarball." % rule)
+        if not self._get_rule_source(source_dir, rule, {
+                "ORIG_VERSION": version,
+                "ORIG_PACKAGE": package}):
+            rule = "get-orig-source"
+            note("Trying to use %s to retrieve needed tarball (deprecated)." % rule)
+            if not self._get_rule_source(source_dir, rule):
+                return None
+        filenames = gather_orig_files(package, version, source_dir)
+        if not filenames:
+            note("%s did not create file for %s version %s", rule, package, version)
+            return None
+        if rule == "get-orig-source":
+            warning("Using get-orig-source to retrieve the packaged orig tarball "
+                    "is deprecated (see debian policy section 4.9). Provide the "
+                    "'get-packaged-orig-source' target instead.")
+        ret = []
+        for filename in filenames:
+            repack_tarball(
+                filename, os.path.basename(filename), target_dir=target_dir)
+            ret.append(os.path.join(target_dir, os.path.basename(filename)))
+        return ret
+
+    def _get_rules_id(self):
         if self.top_level:
             rules_name = 'rules'
         else:
             rules_name = 'debian/rules'
-        rules_id = self.tree.path2id(rules_name)
-        if rules_id is not None:
-            tmpdir = tempfile.mkdtemp(prefix="builddeb-get-orig-source-")
-            try:
-                base_export_dir = os.path.join(tmpdir, "export")
-                export_dir = base_export_dir
-                if self.top_level:
-                    os.mkdir(export_dir)
-                    export_dir = os.path.join(export_dir, "debian")
-                export(self.tree, export_dir, format="dir")
-                tarball_paths = self._get_orig_source(base_export_dir,
-                        "%s_%s.orig" % (package, version), target_dir)
-                if tarball_paths is None:
-                    raise PackageVersionNotPresent(package, version, self)
-                return tarball_paths
-            finally:
-                shutil.rmtree(tmpdir)
-        note("No debian/rules file to try and use for a get-orig-source rule")
-        raise PackageVersionNotPresent(package, version, self)
+        return self.tree.path2id(rules_name)
+
+    def fetch_tarballs(self, package, version, target_dir, components=None):
+        rules_id = self._get_rules_id()
+        if rules_id is None:
+            note("No debian/rules file to use to retrieve upstream tarball.")
+            raise PackageVersionNotPresent(package, version, self)
+        tmpdir = tempfile.mkdtemp(prefix="builddeb-get-source-")
+        try:
+            base_export_dir = os.path.join(tmpdir, "export")
+            export_dir = base_export_dir
+            if self.top_level:
+                os.mkdir(export_dir)
+                export_dir = os.path.join(export_dir, "debian")
+            export(self.tree, export_dir, format="dir")
+            tarball_paths = self._get_current_source(base_export_dir,
+                    package, version, target_dir)
+            if tarball_paths is None:
+                raise PackageVersionNotPresent(package, version, self)
+            return tarball_paths
+        finally:
+            shutil.rmtree(tmpdir)
 
 
 class UScanSource(UpstreamSource):
@@ -279,7 +300,7 @@ class UScanSource(UpstreamSource):
             os.unlink(tempfilename)
         return self._xml_report_extract_upstream_version(stdout)
 
-    def fetch_tarballs(self, package, version, target_dir):
+    def fetch_tarballs(self, package, version, target_dir, components=None):
         note("Using uscan to look for the upstream tarball.")
         try:
             tempfilename = self._export_watchfile()
@@ -326,7 +347,7 @@ class SelfSplitSource(UpstreamSource):
         finally:
             shutil.rmtree(tmpdir)
 
-    def fetch_tarballs(self, package, version, target_dir):
+    def fetch_tarballs(self, package, version, target_dir, components=None):
         note("Using the current branch without the 'debian' directory "
                 "to create the tarball")
         tarball_path = self._tarball_path(package, version, None, target_dir)
@@ -346,10 +367,11 @@ class StackedUpstreamSource(UpstreamSource):
     def __repr__(self):
         return "%s(%r)" % (self.__class__.__name__, self._sources)
 
-    def fetch_tarballs(self, package, version, target_dir):
+    def fetch_tarballs(self, package, version, target_dir, components=None):
         for source in self._sources:
             try:
-                paths = source.fetch_tarballs(package, version, target_dir)
+                paths = source.fetch_tarballs(package, version, target_dir,
+                    components)
             except PackageVersionNotPresent:
                 pass
             else:
@@ -368,7 +390,13 @@ class StackedUpstreamSource(UpstreamSource):
 
 
 def gather_orig_files(package, version, path):
-    prefix = "%s_%s.orig" % (package, version)
+    """Grab the orig files for a particular package.
+
+    :param package: package name
+    :param version: package upstream version string
+    :return: List of orig tarfile paths, or None if none were found
+    """
+    prefix = "%s_%s.orig" % (package.encode('ascii'), version.encode('ascii'))
     ret = []
     path = os.path.abspath(path)
     if not os.path.isdir(path):
@@ -414,7 +442,7 @@ class UpstreamProvider(object):
              retrieved.
            - Else if uscan knows about that version it will be downloaded
              and repacked as needed.
-           - Else a call will be made to get-orig-source to try and retrieve
+           - Else a call will be made to debian/rules to try and retrieve
              the tarball.
 
         If the tarball can't be found at all then MissingUpstreamTarball
@@ -496,7 +524,7 @@ class TarfileSource(UpstreamSource):
         self.path = path
         self.version = version
 
-    def fetch_tarballs(self, package, version, target_dir):
+    def fetch_tarballs(self, package, version, target_dir, components=None):
         if version != self.version:
             raise PackageVersionNotPresent(package, version, self)
         dest_name = new_tarball_name(package, version, self.path)
@@ -541,7 +569,7 @@ class LaunchpadReleaseFileSource(UpstreamSource):
         else:
             self.project = project
 
-    def fetch_tarballs(self, package, version, target_dir):
+    def fetch_tarballs(self, package, version, target_dir, components=None):
         release = self.project.getRelease(version=version)
         if release is None:
             raise PackageVersionNotPresent(package, version, self)
@@ -557,7 +585,7 @@ class LaunchpadReleaseFileSource(UpstreamSource):
             warning("More than one release file for release %s of package %s"
                     "found on Launchpad. Using the first.", version, package)
         hosted_file = release_files[0]
-        tmpdir = tempfile.mkdtemp(prefix="builddeb-get-orig-source-")
+        tmpdir = tempfile.mkdtemp(prefix="builddeb-launchpad-source-")
         try:
             inf = hosted_file.open()
             try:
