@@ -29,7 +29,6 @@ import subprocess
 import tempfile
 
 from bzrlib.plugins.builddeb.errors import (
-    MultipleUpstreamTarballsNotSupported,
     PackageVersionNotPresent,
     PerFileTimestampsNotSupported,
     )
@@ -39,13 +38,17 @@ from bzrlib.plugins.builddeb.util import (
     subprocess_setup,
     )
 
-from bzrlib import osutils
+from bzrlib import (
+    osutils,
+    revision as _mod_revision,
+    )
 from bzrlib.errors import (
     BzrError,
     NoSuchRevision,
     NoSuchTag,
     )
 from bzrlib.trace import (
+    mutter,
     note,
     warning,
     )
@@ -156,42 +159,35 @@ class PristineTarSource(UpstreamSource):
         self.branch.tags.set_tag(tag_name, revid)
         return tag_name, revid
 
-    def import_tarballs(self, package, version, tree, parent_ids, tarballs,
-            timestamp=None, author=None):
-        """Import the upstream tarballs.
-
-        :param package: Package name
-        :param version: Package version
-        :param path: Path with tree to import
-        :param parent_ids: Parent revisions
-        :param tarballs: List of (path, component, md5)
-        :param timestamp: Optional timestamp for new commits
-        :param author: Optional author for new commits
-        :return: List of tuples with (component, tag, revid)
-        """
-        ret = []
-        for (tarball, component, md5) in tarballs:
-            (tag, revid) = self.import_component_tarball(
-                package, version, tree, parent_ids, component,
-                md5, tarball, author=author, timestamp=timestamp)
-            ret.append((component, tag, revid))
-        return ret
-
     def import_component_tarball(self, package, version, tree, parent_ids,
-            component=None, md5=None, tarball=None, author=None, timestamp=None):
+            component=None, md5=None, tarball=None, author=None, timestamp=None,
+            subdir=None, exclude=None):
         """Import a tarball.
 
         :param package: Package name
         :param version: Upstream version
+        :param parent_ids: Dictionary mapping component names to revision ids
         :param component: Component name (None for base)
+        :param exclude: Exclude directories
         """
-        if component is not None:
-            raise BzrError("Importing non-base tarballs not yet supported")
-        tree.set_parent_ids(parent_ids)
+        if exclude is None:
+            exclude = []
+        def include_change(c):
+            if not exclude:
+                return True
+            path = c[1][1]
+            if path is None:
+                return True
+            for e in exclude:
+                if path == e or path.startswith(e+"/"):
+                    return False
+            else:
+                return True
         revprops = {}
         if md5 is not None:
             revprops["deb-md5"] = md5
-            delta = self.make_pristine_tar_delta(tree, tarball)
+            delta = self.make_pristine_tar_delta(tree, tarball, subdir=subdir,
+                    exclude=exclude)
             uuencoded = standard_b64encode(delta)
             if tarball.endswith(".tar.bz2"):
                 revprops["deb-pristine-delta-bz2"] = uuencoded
@@ -208,9 +204,33 @@ class PristineTarSource(UpstreamSource):
         message = "Import upstream version %s" % (version,)
         if component is not None:
             message += ", component %s" % component
-        revid = tree.commit(message, revprops=revprops, timestamp=timestamp,
-            timezone=timezone)
-        tag_name, _ = self.tag_version(version, revid=revid)
+            revprops["deb-component"] = component
+        if len(parent_ids) == 0:
+            base_revid = _mod_revision.NULL_REVISION
+        else:
+            base_revid = parent_ids[0]
+        basis_tree = tree.branch.repository.revision_tree(base_revid)
+        tree.lock_write()
+        try:
+            builder = tree.branch.get_commit_builder(parents=parent_ids,
+                revprops=revprops, timestamp=timestamp, timezone=timezone)
+            builder.will_record_deletes()
+            try:
+                changes = [c for c in tree.iter_changes(basis_tree) if
+                           include_change(c)]
+                list(builder.record_iter_changes(tree, base_revid, changes))
+                builder.finish_inventory()
+            except:
+                builder.abort()
+                raise
+            revid = builder.commit(message)
+            tag_name, _ = self.tag_version(version, revid=revid, component=component)
+            tree.update_basis_by_delta(revid, builder.get_basis_delta())
+        finally:
+            tree.unlock()
+        mutter(
+            'imported %s version %s component %r as revid %s, tagged %s',
+            package, version, component, revid, tag_name)
         return tag_name, revid
 
     def fetch_component_tarball(self, package, version, component, target_dir):
@@ -219,13 +239,13 @@ class PristineTarSource(UpstreamSource):
             rev = self.branch.repository.get_revision(revid)
         except NoSuchRevision:
             raise PackageVersionNotPresent(package, version, self)
-        note("Using pristine-tar to reconstruct the needed tarball.")
         if self.has_pristine_tar_delta(rev):
             format = self.pristine_tar_format(rev)
         else:
             format = 'gz'
         target_filename = self._tarball_path(package, version, component,
                                              target_dir, format=format)
+        note("Using pristine-tar to reconstruct %s.", os.path.basename(target_filename))
         try:
             self.reconstruct_pristine_tar(revid, package, version, target_filename)
         except PristineTarError:
@@ -234,8 +254,15 @@ class PristineTarSource(UpstreamSource):
             raise PackageVersionNotPresent(package, version, self)
         return target_filename
 
-    def fetch_tarballs(self, package, version, target_dir):
-        return [self.fetch_component_tarball(package, version, None, target_dir)]
+    def fetch_tarballs(self, package, version, target_dir, components=None):
+        if components is None:
+            # Scan tags for components
+            try:
+                components = self._components_by_version()[version].keys()
+            except KeyError:
+                raise PackageVersionNotPresent(package, version, self)
+        return [self.fetch_component_tarball(package, version, component, target_dir)
+                for component in components]
 
     def _has_revision(self, revid, md5=None):
         self.branch.lock_read()
@@ -256,14 +283,16 @@ class PristineTarSource(UpstreamSource):
                 "revision %s", revid)
             return True
 
-    def version_as_revision(self, package, version, tarballs=None):
+    def version_as_revisions(self, package, version, tarballs=None):
         if tarballs is None:
-            return self.version_component_as_revision(package, version, component=None)
-        elif len(tarballs) > 1:
-            raise MultipleUpstreamTarballsNotSupported()
-        else:
-            return self.version_component_as_revision(package, version, tarballs[0][1],
-                tarballs[0][2])
+            # FIXME: What if there are multiple tarballs?
+            return {
+                None: self.version_component_as_revision(package, version, component=None) }
+        ret = {}
+        for (tarball, component, md5) in tarballs:
+            ret[component] = self.version_component_as_revision(
+                package, version, component, md5)
+        return ret
 
     def version_component_as_revision(self, package, version, component, md5=None):
         assert isinstance(version, str)
@@ -284,11 +313,11 @@ class PristineTarSource(UpstreamSource):
     def has_version(self, package, version, tarballs=None):
         if tarballs is None:
             return self.has_version_component(package, version, component=None)
-        elif len(tarballs) > 1:
-            raise MultipleUpstreamTarballsNotSupported()
         else:
-            return self.has_version_component(package, version, tarballs[0][1],
-                    tarballs[0][2])
+            for (tarball, component, md5) in tarballs:
+                if not self.has_version_component(package, version, component, md5):
+                    return False
+            return True
 
     def has_version_component(self, package, version, component, md5=None):
         assert isinstance(version, str), str(type(version))
@@ -311,6 +340,8 @@ class PristineTarSource(UpstreamSource):
         if component is None:
             # compatibility with git-buildpackage
             tags += ["upstream/%s" % version]
+            # compatibility with svn-buildpackage
+            tags += ["upstream_%s" % version]
         return tags
 
     def has_pristine_tar_delta(self, rev):
@@ -359,7 +390,7 @@ class PristineTarSource(UpstreamSource):
         finally:
             shutil.rmtree(tmpdir)
 
-    def make_pristine_tar_delta(self, tree, tarball_path):
+    def make_pristine_tar_delta(self, tree, tarball_path, subdir=None, exclude=None):
         tmpdir = tempfile.mkdtemp(prefix="builddeb-pristine-")
         try:
             dest = os.path.join(tmpdir, "orig")
@@ -367,22 +398,29 @@ class PristineTarSource(UpstreamSource):
             try:
                 for (dp, ie) in tree.inventory.iter_entries():
                     ie._read_tree_state(dp, tree)
-                export(tree, dest, format='dir')
+                export(tree, dest, format='dir', subdir=subdir)
             finally:
                 tree.unlock()
             return make_pristine_tar_delta(dest, tarball_path)
         finally:
             shutil.rmtree(tmpdir)
 
+    def _components_by_version(self):
+        ret = {}
+        for tag_name, tag_revid in self.branch.tags.get_tag_dict().iteritems():
+            if not is_upstream_tag(tag_name):
+                continue
+            (component, version) = upstream_tag_version(tag_name)
+            ret.setdefault(version, {})[component] = tag_revid
+        return ret
+
     def iter_versions(self):
         """Iterate over all upstream versions.
 
         :return: Iterator over (tag_name, version, revid) tuples
         """
-        for tag_name, tag_revid in self.branch.tags.get_tag_dict().iteritems():
-            if not is_upstream_tag(tag_name):
-                continue
-            yield (tag_name, upstream_tag_version(tag_name), tag_revid)
+        ret = self._components_by_version()
+        return ret.iteritems()
 
 
 def is_upstream_tag(tag):
@@ -391,22 +429,30 @@ def is_upstream_tag(tag):
     :param tag: The string name of the tag.
     :return: True if the tag name is one generated by upstream tag operations.
     """
-    return tag.startswith('upstream-') or tag.startswith('upstream/')
+    return (tag.startswith('upstream-') or tag.startswith('upstream/') or
+            tag.startswith('upstream_'))
 
 
 def upstream_tag_version(tag):
     """Return the upstream version portion of an upstream tag name.
 
     :param tag: The string name of the tag.
-    :return: The version portion of the tag.
+    :return: tuple with version portion of the tag and component name
     """
     assert is_upstream_tag(tag), "Not an upstream tag: %s" % tag
     if tag.startswith('upstream/'):
         tag = tag[len('upstream/'):]
+    elif tag.startswith('upstream_'):
+        tag = tag[len('upstream_'):]
     elif tag.startswith('upstream-'):
         tag = tag[len('upstream-'):]
         if tag.startswith('debian-'):
             tag = tag[len('debian-'):]
         elif tag.startswith('ubuntu-'):
             tag = tag[len('ubuntu-'):]
-    return tag
+    if not '/' in tag:
+        return (None, tag)
+    (version, component) = tag.rsplit('/', 1)
+    if component == "":
+        component = None
+    return (component, version)
