@@ -29,6 +29,8 @@ import tempfile
 import os
 import re
 
+from distro_info import DebianDistroInfo, UbuntuDistroInfo
+
 from bzrlib.trace import mutter
 
 try:
@@ -55,6 +57,8 @@ from bzrlib.plugins.builddeb import (
     default_conf,
     local_conf,
     global_conf,
+    new_conf,
+    new_local_conf,
     )
 from bzrlib.plugins.builddeb.config import (
     DebBuildConfig,
@@ -73,12 +77,10 @@ from bzrlib.plugins.builddeb.errors import (
     )
 
 
-DEBIAN_RELEASES = ('woody', 'sarge', 'etch', 'lenny', 'squeeze', 'wheezy',
-        'stable', 'testing', 'unstable', 'experimental', 'frozen', 'sid')
+DEBIAN_RELEASES = DebianDistroInfo().all
+DEBIAN_RELEASES.extend(['stable', 'testing', 'unstable', 'frozen'])
 DEBIAN_POCKETS = ('', '-security', '-proposed-updates', '-backports')
-UBUNTU_RELEASES = ('warty', 'hoary', 'breezy', 'dapper', 'edgy',
-        'feisty', 'gutsy', 'hardy', 'intrepid', 'jaunty', 'karmic',
-        'lucid', 'maverick', 'natty', "oneiric")
+UBUNTU_RELEASES = UbuntuDistroInfo().all
 UBUNTU_POCKETS = ('', '-proposed', '-updates', '-security', '-backports')
 
 
@@ -533,14 +535,25 @@ def debuild_config(tree, working_tree):
     """
     config_files = []
     user_config = None
+    if (working_tree and tree.has_filename(new_local_conf)):
+        if tree.path2id(new_local_conf) is None:
+            config_files.append((tree.get_file_byname(new_local_conf), True,
+                        "local.conf"))
+        else:
+            warning('Not using configuration from %s as it is versioned.',
+                    new_local_conf)
     if (working_tree and tree.has_filename(local_conf)):
         if tree.path2id(local_conf) is None:
             config_files.append((tree.get_file_byname(local_conf), True,
                         "local.conf"))
         else:
-            warning('Not using configuration from %s as it is versioned.')
+            warning('Not using configuration from %s as it is versioned.',
+                    local_conf)
     config_files.append((global_conf(), True))
     user_config = global_conf()
+    if tree.path2id(new_conf):
+        config_files.append((tree.get_file(tree.path2id(new_conf)), False,
+                    "bzr-builddeb.conf"))
     if tree.path2id(default_conf):
         config_files.append((tree.get_file(tree.path2id(default_conf)), False,
                     "default.conf"))
